/**
 * project: Mobile Datacommunication
 * file: HTTPMidlet.java
 * date: 2003-03-10
 */

package com.maretzke.mobile.http;

// import J2ME related stuff
import javax.microedition.midlet.*;
import javax.microedition.lcdui.*;
import javax.microedition.io.*;

// import java related stuff
import java.io.*;

/**
 * A MIDlet establishing a HTTP connection to a HTTP server 
 * fetching a page using HTTP get. 
 *
 * @author Michael Maretzke
 * @version 1.0
 */
public class HTTPMIDlet extends MIDlet implements CommandListener {
  // "exit" command option
  private Command exitCommand;
  // "get" command option
  private Command getCommand;
  // display for this MIDlet
  private Display display;
  
  /**
   * Default Constructor for MIDlet
   */
  public HTTPMIDlet() {
    display = Display.getDisplay(this);
    getCommand = new Command("Get", Command.SCREEN, 1);
    exitCommand = new Command("Exit", Command.SCREEN, 2);
  }
  
  /**
   * Start up the MIDlet by creating the TextBox and associating
   * the exit and get command and listener.
   */
  public void startApp() {
    TextBox textbox = new TextBox("HTTP MIDlet", "http://www.maretzke.de", 256, 0);
    
    textbox.addCommand(exitCommand);
    textbox.addCommand(getCommand);
    textbox.setCommandListener(this);
    
    display.setCurrent(textbox);
  }
  
  /**
   * Pause is a no-op since there are no background activities or
   * record stores that need to be closed.
   */
  public void pauseApp() {
  }
  
  /**
   * Destroy must cleanup everything not handled by the garbage collector.
   * In this case there is nothing to cleanup.
   */
  public void destroyApp(boolean unconditional) {
  }
  
  /*
   * Respond to commands, including exit
   * On the exit command, cleanup and notify that the MIDlet has been destroyed.
   */
  public void commandAction(Command command, Displayable display) {
    // exit application
    if (command == exitCommand) {
      destroyApp(false);
      notifyDestroyed();
    }
    // get the URL
    if (command == getCommand) {
      if (display instanceof TextBox) {
        TextBox textbox = (TextBox) display;
        System.out.println("Fetching URL: " + textbox.getString());
        try {
          // fetch the page
          getPage(textbox.getString());
        }
        catch (IOException ioe) {
          System.out.println("Caught IOException: " + ioe);
        }
      }
      
    }
  }
  
  /**
   * Get a URL from a HTTP server instance
   * @param url the url to get
   * @exception IOException is thrown if anything during communication went wrong
   */
  public void getPage(String url) throws IOException {
    // maximum amount of communication retries
    final int HTTP_MAX_RETRIES = 15;
 
    // http connection to use for communication
    HttpConnection connection = null;
    
    // attached input stream
    InputStream input = null;
    
    // flag indicating whether communication succeeds or fails
    boolean commOk = false;
    
    // counter for communication retries
    int commCounter = 0;
      
    while ((!commOk) && (commCounter < HTTP_MAX_RETRIES)) {
      try {
        System.out.println("Opening URL.");
        // Connector is a placeholder for the static methods that are 
        // used for creating all the Connection objects
        connection = (HttpConnection) Connector.open(url);
   
        // Getting the InputStream will open the connection and read the HTTP headers
        System.out.println("Getting input stream.");
        input = connection.openInputStream();
   
        // Get the length and process the data
        System.out.println("Get the length and process the data.");
        int length = (int) connection.getLength();
          if (length > 0) {
            byte[] data = new byte[length];
            int read = input.read(data);
            System.out.println(new String(data));
          }
        commOk = true;
        commCounter = 0;
         
      } 
      catch(IOException ioe) {
        if (commCounter == HTTP_MAX_RETRIES) 
          throw ioe;
        commCounter++;
        System.out.println(commCounter + ". Retry ... Caught an IOException: " + ioe);
      }
      finally {
        if (input != null)
          input.close();
        if (connection != null)
          connection.close();
      }
      System.out.println("Finished.");
    }
  }
}
  
