/**
 * project: Mobile Datacommunication
 * file: JMSReceiver.java
 * date: 2003-03-09
 */

package com.maretzke.mobile.jms;

// import java related classes
import java.util.Hashtable;

// import JMS related classes
import javax.jms.Queue;
import javax.jms.QueueConnection;
import javax.jms.QueueSession;
import javax.jms.QueueReceiver;
import javax.jms.QueueConnectionFactory;
import javax.jms.Message;
import javax.jms.TextMessage;
import javax.jms.JMSException;

// import naming related classes
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

/**
 * JMSReceiver receives messages from a queue. Therefore it starts listening on 
 * queue "ConnectionTestQueue". Another java class com.maretzke.mobile.jms.JMSSender 
 * sends simple textmessages into this queue. 
 * This implementation will use JoRAM - a freeware JMS implementation.
 * The queue itself is already prepared and configured in JoRAM.
 *
 * @author Michael Maretzke
 * @version 1.0
 */
public class JMSReceiver {
  /**
   * Main method of JMSReceiver
   * @param args command line input parameters
   */
  public static void main(String [] args) {
    // maximum amount of communication retries
    final int JMS_MAX_RETRIES = 5;
    
    // queue to use for communication
    Queue queue = null; 

    // connection factory to build connection objects
    QueueConnectionFactory qcf = null;
    
    // queue connection to use for communication
    QueueConnection qc;
    
    // session used during communication
    QueueSession qs;
    
    // receiver getting messages
    QueueReceiver qrec;    
    
    // message received
    Message msg;
    
    // flag indicating whether communication succeeds or fails
    boolean commOk = false;
    
    // counter for communication retries
    int commCounter = 1;
    
    System.out.println("Start JMS Receiver ...");

    // create a initial jndi context
		Hashtable env = new Hashtable();
		env.put(Context.INITIAL_CONTEXT_FACTORY, "fr.dyade.aaa.jndi.NamingContextFactory");
		env.put("java.naming.factory.host", "localhost");
		env.put("java.naming.factory.port", "16400");		
      
    while ((!commOk) && (commCounter < JMS_MAX_RETRIES)) {
      try {      
        System.out.println("JNDI Lookup [" + commCounter + "].");
        
        Context ctx = new InitialContext(env);
      
        // Lookup the queue in JNDI space
        queue = (Queue) ctx.lookup("ConnectionTestQueue");
        // Lookup the queue connection factory in JNDI
        qcf = (QueueConnectionFactory) ctx.lookup("ConnectionTestQueueFactory");

        ctx.close();
    
        System.out.println("JMS Communication [" + commCounter + "].");
        // create a new connection to the queue
        qc = qcf.createQueueConnection();
        // create a session 
        qs = qc.createQueueSession(true, 0);
        // create the receiver and start receiving
        qrec = qs.createReceiver(queue);
        qc.start();

        int i;
        for (i = 0; i < 10; i++) {
          msg = qrec.receive();
          if (msg instanceof TextMessage)
            System.out.println("Msg received: " + ((TextMessage) msg).getText());
        }

        // commit whole session
        qs.commit();

        System.out.println(i + " messages received.");

        // free resources ...
        qs.close();
        qc.close();
        // ... and quit
        commOk = true;
        commCounter = 1;
      }
      catch (JMSException je) {
        commCounter++;
        System.out.println(commCounter + ". Retry ... Caught a JMSException: " + je);     
      }    
      catch (NamingException ne) {
        commCounter++;
        System.out.println(commCounter + ". Retry ... Caught a NamingException: " + ne);
      }
    }   
    if (!commOk)
      System.out.println("Unable to complete Communication. Aborting.");
    else
      System.out.println("Communication successfully ended.");    
  }
}
