/**
 * project: Mobile Datacommunication
 * file: RMIClient.java
 * date: 2003-03-09
 */

package com.maretzke.mobile.rmi;

// import RMI related stuff
import java.rmi.Naming;
import java.rmi.RemoteException;

/**
 * The client implementation of RMI test. This will call the remote method "echo"
 * implemented by com.maretzke.mobile.rmi.StringServerImpl.
 *
 * @author Michael Maretzke
 * @version 1.0
 */
public class RMIClient {

  /**
   * Main method of RMIClient
   * @param args command line input parameters
   */	
  public static void main(String[] args) {
    // maximum amount of communication retries
    final int RMI_MAX_RETRIES = 15;
    // number of RMI calls to do during execution
    final int RMI_CALLS = 1000;
    
    // message to send
		String smessage = "Mobile Datacommunication"; 

    // reference to StringServerImpl
	  StringServer ssref = null; 
	
    // message received
    String rmessage = "";    
    
    // flag indicating whether communication succeeds or fails
    boolean commOk = false;
    
    // counter for communication retries
    int commCounter = 0;
    
    while ((!commOk) && (commCounter < RMI_MAX_RETRIES)) {      
      try { 
        // look up the reference
        ssref = (StringServer) Naming.lookup ("//localhost/StringServer"); 

        for (int i = 0; i < RMI_CALLS; i++) {
          // do the remote call
          rmessage = ssref.echo(smessage);
          // reset commCounter
          commCounter = 0;

          System.out.println(i + ". Send message was: " + smessage);
          System.out.println(i + ". Received message is: " + rmessage);
        }
        commOk = true;
      } 
      catch (RemoteException re) {
        commCounter++;
        System.out.println(commCounter + ". Retry ... Remoteexception: " + re);
      }
      catch (Exception e) { 
        commCounter++;
        System.out.println(commCounter + ". Retry ... RMIClient exception: " + e); 
      } 
    }
    if (!commOk)
      System.out.println("Unable to complete Communication. Aborting.");
    else
      System.out.println("Communication successfully ended.");    
  }
}